@@ -1,4 +1,12 @@
 """
+.. note::
+
+    While modeltree is functional and well tested, it's in an early state of
+    its development. Backward incompatible api changes are possible. Feedback
+    and suggestions about the api are very welcome. Just open an issue on
+    github.
+
+
 What is a ModelTree?
 --------------------
 
@@ -43,16 +51,16 @@ Then a tree representing these models will look like::
     >>> tree = ModelTree(ModelOne)
     >>> tree.show()
     ModelOne
-    └── [many_to_many] ModelOne.model_two => ModelTwo
-        └── [many_to_one] ModelTwo.model_three => ModelThree
-            ├── [one_to_one] ModelThree.model_four => ModelFour
-            └── [many_to_many] ModelThree.model_five => ModelFive
+    └── model_two -> ModelTwo
+        └── model_three -> ModelThree
+            ├── model_four -> ModelFour
+            └── model_five -> ModelFive
 
 Or rendered by using the :attr:`~.ModelTree.field_path` attribute::
 
     >>> tree = ModelTree(ModelOne)
-    >>> tree.show('field_path')
-    root
+    >>> tree.show('{node.field_path}')
+    ModelOne
     └── model_two
         └── model_two__model_three
             ├── model_two__model_three__model_four
@@ -78,7 +86,7 @@ with which you initiated your tree::
 
     >>> items = ModelOne.objects.all()
     >>> tree = ModelTree(ModelOne, items)
-    >>> model_five_node = tree.get(model=ModelFive)
+    >>> model_five_node = tree.get(filter=lambda n: n.model == ModelFive)
     >>> len(model_five_node.items)
     0
 
@@ -90,9 +98,10 @@ What if I don't want to follow all model relations?
 ---------------------------------------------------
 
 You can easily adjust the way your tree is build up. Therefore overwrite one
-or more of the following class attributes:
+or more of the following class attributes in your ModelTree subclass:
 
 * :attr:`~.ModelTree.MAX_DEPTH`
+* :attr:`~.ModelTree.FOLLOW_ACROSS_APPS`
 * :attr:`~.ModelTree.RELATION_TYPES`
 * :attr:`~.ModelTree.FIELD_TYPES`
 * :attr:`~.ModelTree.FIELD_PATHS`
@@ -108,11 +117,15 @@ Guess you whish to only follow specific relation-types::
     >>> tree = MyModelTree(ModelOne)
     >>> tree.show()
     ModelOne
-    └── [many_to_many] ModelOne.model_two => ModelTwo
-        └── [many_to_one] ModelTwo.model_three => ModelThree
-            └── [many_to_many] ModelThree.model_five => ModelFive
+    └── model_two -> ModelTwo
+        └── model_three -> ModelThree
+            └── model_five -> ModelFive
+
+For further adjustments you could also overwrite the private
+:meth:`~.ModelTree._follow` method. See the description below.
 """
 
+from django.db import models
 from anytree import AnyNode
 from anytree import RenderTree
 from anytree import LevelOrderIter
@@ -122,6 +135,22 @@ from anytree import find
 from anytree import findall
 
 
+RELATION_TYPES = [
+    'one_to_one',
+    'one_to_many',
+    'many_to_one',
+    'many_to_many',
+]
+FIELD_TYPES = [
+    models.OneToOneField,
+    models.OneToOneRel,
+    models.ForeignKey,
+    models.ManyToOneRel,
+    models.ManyToManyField,
+    models.ManyToManyRel,
+]
+
+
 class ModelTree(AnyNode):
     """
     A ModelTree is technical a Subclass of :class:`~anytree.node.node.AnyNode`,
@@ -132,10 +161,10 @@ class ModelTree(AnyNode):
         >>> tree = ModelTree(ModelOne)
         >>> tree.show()
         ModelOne
-        └── [many_to_many] ModelOne.model_two => ModelTwo
-            └── [many_to_one] ModelTwo.model_three => ModelThree
-                ├── [one_to_one] ModelThree.model_four => ModelFour
-                └── [many_to_many] ModelThree.model_five => ModelFive
+        └── model_two -> ModelTwo
+            └── model_three -> ModelThree
+                ├── model_four -> ModelFour
+                └── model_five -> ModelFive
 
     In advance you can pass in some items of your model as a queryset. The
     :attr:`.items` property of each node of your tree then reflects the itmes
@@ -150,7 +179,7 @@ class ModelTree(AnyNode):
 
         >>> items = ModelOne.objects.all()
         >>> tree = ModelTree(ModelOne, items)
-        >>> model_four_node = tree.get(model=ModelFour)
+        >>> model_four_node = tree.get(filter=lambda n: n.model == ModelFour)
         >>> len(model_four_node.items)
         0
 
@@ -162,13 +191,18 @@ class ModelTree(AnyNode):
 
     MAX_DEPTH = 3
     """
-    Max depth of the tree structure
+    Max depth of the tree structure.
     """
 
-    RELATION_TYPES = None
+    FOLLOW_ACROSS_APPS = False
     """
-    A list of relations types as strings to follow when building the tree.
-    By default all relations types will be followed. Types might be::
+    Follow relations across different apps.
+    """
+
+    RELATION_TYPES = RELATION_TYPES
+    """
+    A list of relation-types as strings to follow when building the tree.
+    By default all relation-types will be followed::
 
         RELATION_TYPES = [
             'one_to_one',
@@ -178,10 +212,10 @@ class ModelTree(AnyNode):
         ]
     """
 
-    FIELD_TYPES = None
+    FIELD_TYPES = FIELD_TYPES
     """
-    A list of field types to follow when building the tree.
-    By default all field types will be followed. Types might be::
+    A list of field-types to follow when building the tree.
+    By default all field-types and their reverse field-types are followed::
 
         FIELD_TYPES = [
             models.OneToOneField,
@@ -191,6 +225,10 @@ class ModelTree(AnyNode):
             models.ManyToManyField,
             models.ManyToManyRel,
         ]
+
+    .. note::
+    
+        Generic relations using the contenttypes framework are not supported.
     """
 
     FIELD_PATHS = None
@@ -219,28 +257,15 @@ class ModelTree(AnyNode):
         self._items = items
         self._build_tree()
 
+    def __str__(self):
+        if self._field:
+            return '{} -> {}'.format(self._field.name, self._model._meta.object_name)
+        else:
+            return '{}'.format(self._model._meta.object_name)
+
     def __repr__(self):
         classname = type(self).__name__
-        return '{}(model={}, field={})'.format(classname, self.model, self.field)
-
-    @property
-    def name(self):
-        """
-        A unique name as a identifier of the node. Since the :attr:`.field_path`
-        is the most compact and unique value describing a node it serves best
-        as the node's name attribute::
-
-            >>> node = tree.get('model_two__model_three')
-            >>> node.name == node.field_path
-            True
-
-        The name of the root node which has no field by its own is simply the
-        string 'root'::
-
-            >>> tree.root.name
-            'root'
-        """
-        return self.field_path
+        return '{}(model={}, field={})'.format(classname, repr(self._model), repr(self._field))
 
     @property
     def model(self):
@@ -249,13 +274,6 @@ class ModelTree(AnyNode):
         """
         return self._model
 
-    @property
-    def model_name(self):
-        """
-        Class name of the node's :attr:`.model`.
-        """
-        return self.model._meta.object_name
-
     @property
     def field(self):
         """
@@ -264,22 +282,6 @@ class ModelTree(AnyNode):
         """
         return self._field
 
-    @property
-    def field_name(self):
-        """
-        Name attribute of the node's :attr:`.field`.
-        This is an empty string for the root node.
-        """
-        return self.field.name if self.field else str()
-
-    @property
-    def field_type(self):
-        """
-        Type of the :attr:`.field`.
-        This is None for the root node.
-        """
-        return type(self.field) if self.field else None
-
     @property
     def relation_type(self):
         """
@@ -288,78 +290,10 @@ class ModelTree(AnyNode):
         This is an empty string for the root node.
         """
         if self.field:
-            relation_types = ['one_to_one', 'one_to_many', 'many_to_one', 'many_to_many']
-            return [t for t in relation_types if getattr(self.field, t)][0]
+            return [t for t in RELATION_TYPES if getattr(self.field, t)][0]
         else:
             return str()
 
-    @property
-    def label(self):
-        """
-        String describing the field-model relation::
-
-            >>> node_two = tree.get('model_two')
-            >>> node_two.label
-            'model_two -> ModelTwo'
-
-        Since the root node has no field assigned it only shows the class name
-        of its model::
-
-            >>> tree.root.label
-            'ModelOne'
-        """
-        if self.field:
-            return '{} -> {}'.format(self.field.name, self.model_name)
-        else:
-            return str(self.model_name)
-
-    @property
-    def verbose_label(self):
-        """
-        String describing the field-model relation including the relation-type
-        and the parent-model::
-
-            >>> node_two = tree.get('model_two')
-            >>> node_two.verbose_label
-            '[many_to_many] ModelOne.model_two => ModelTwo'
-
-        Since the root node has no field assigned it only shows the class name
-        of its model::
-
-            >>> tree.root.verbose_label
-            'ModelOne'
-        """
-        if self.field:
-            relation_types = ['one_to_one', 'one_to_many', 'many_to_one', 'many_to_many']
-            relation_type = [t for t in relation_types if getattr(self.field, t)][0]
-            return '[{}] {}.{} => {}'.format(relation_type, self.parent.model_name, self.field.name, self.model_name)
-        else:
-            return str(self.model_name)
-
-    @property
-    def label_path(self):
-        """
-        String describing the node's :attr:`~anytree.node.nodemixin.NodeMixin.path`
-        using the :attr:`.label` of nodes::
-
-            >>> node_three = list(tree.iterate())[2]
-            >>> node_three.label_path
-            'ModelOne.model_two -> ModelTwo.model_three -> ModelThree'
-        """
-        return '.'.join(n.label for n in self.path)
-
-    @property
-    def model_path(self):
-        """
-        String describing the node's :attr:`~anytree.node.nodemixin.NodeMixin.path`
-        using the :attr:`.model_name` of nodes::
-
-            >>> node_three = list(tree.iterate())[2]
-            >>> node_three.model_path
-            'ModelOne -> ModelTwo -> ModelThree'
-        """
-        return ' -> '.join(n.model_name for n in self.path)
-
     @property
     def field_path(self):
         """
@@ -375,6 +309,7 @@ class ModelTree(AnyNode):
 
             >>> tree.root.field_path
             'root'
+
         """
         if self.is_root:
             return 'root'
@@ -420,68 +355,62 @@ class ModelTree(AnyNode):
         """
         return RenderTree(self)
 
-    def show(self, key='verbose_label'):
+    def show(self, format='{node}', root_format='{node}'):
         """
-        Print a rendered tree using a specific attribute. 
-
-        :param str key: attribute to be used to render the tree (optional)
-        """
-        print(self.render().by_attr(key))
-
-    def get(self, name=None, **params):
-        """
-        Either lookup a node by its :attr:`.name` or pass in any number of
-        parameter to identify a specific node::
+        Print a tree. Each node will be rendered by using a format string which
+        reference the node object by the key *node*::
 
             >>> tree = ModelTree(ModelOne)
-            >>> tree.get('model_two') == tree.get(model=ModelTwo)
+            >>> tree.show('{node.field.model._meta.object_name}.{node.field.name} -> {node.model._meta.object_name}')
+            ModelOne
+            └── ModelOne.model_two -> ModelTwo
+                └── ModelTwo.model_three -> ModelThree
+                    ├── ModelThree.model_four -> ModelFour
+                    └── ModelThree.model_five -> ModelFive
+
+        :param str format: format string to render a node object (optional)
+        :param str root_format: format string to render the root node object (optional)
+        """
+        for prefix, _, node in self.render():
+            if root_format and node.is_root:
+                label = root_format.format(node=node)
+            else:
+                label = format.format(node=node)
+            print(f'{prefix}{label}')
+
+    def get(self, field_path=None, filter=None):
+        """
+        Either lookup a node by its :attr:`.field_path` or use a filter::
+
+            >>> tree = ModelTree(ModelOne)
+            >>> tree.get('model_two') == tree.get(filter=lambda n: n.model == ModelTwo)
             True
 
-        :param str name: a node's :attr:`.name` (optional)
+        :param str field_path: a node's :attr:`.field_path` (optional)
         :params: any key value pair referencing to a node's attribute
         :return: node object or None if no node matches the search parameters
         :rtype: :class:`~modeltree.ModelTree` or None
         :raises: :class:`~anytree.search.CountError`: if more than one node was found
         """
-        if not name is None:
-            params['name'] = name
-        filter = lambda n: all(getattr(n, k) == v for k, v in params.items())
+        if field_path:
+            filter = lambda n: n.field_path == field_path
         return find(self, filter)
 
-    def find(self, **params):
+    def find(self, filter):
         """
-        Find nodes by specific attributes::
+        Find nodes using a filter::
 
             >>> tree = ModelTree(ModelOne)
-            >>> len(tree.find(relation_type='many_to_many'))
+            >>> len(tree.find(lambda n: n.relation_type == 'many_to_many'))
             2
-            >>> len(tree.find(field_type=models.OneToOneRel))
+            >>> len(tree.find(lambda n: type(n.field) == models.OneToOneRel))
             1
 
         :params: any key value pair referencing to attributes of nodes
         :return: tuple of nodes
         """
-        filter = lambda n: all(getattr(n, k) == v for k, v in params.items())
         return findall(self, filter)
 
-    def grep(self, pattern, key='name'):
-        """
-        Grep nodes by a pattern that matches a specific attribute. The
-        attribute's value must be of type string. By default the :attr:`.name`
-        attribute of a node is used::
-
-            >>> tree = ModelTree(ModelOne)
-            >>> len(tree.grep('model_two'))
-            4
-            >>> len(tree.grep('many_to', key='relation_type'))
-            3
-
-        :param str pattern: a pattern matching a string attribute of the node
-        :param str key: name of the attribute to be tried
-        :return: tuple of nodes
-        """
-        return findall(self, lambda n: pattern in getattr(n, key))
-
     def iterate(self, by_level=False, by_grouped_level=False, maxlevel=None, has_items=False, filter=None):
         """
         Return a tree iterator using the iteration classes of anytree:
@@ -499,7 +428,7 @@ class ModelTree(AnyNode):
         :param callable filter: a callable recieving the node and returning a boolean.
         :return: iterator
         """
-        filters = list(filter) if filter else list()
+        filters = [filter] if filter else list()
         if has_items:
             filters.append(lambda n: bool(n.items))
 
@@ -515,6 +444,30 @@ class ModelTree(AnyNode):
 
         return iter_class(self, maxlevel=maxlevel, filter_=filter)
 
+    def _follow(self, field):
+        """
+        To fine-tune the way a tree is build overwrite this method. You can do
+        what ever you want evaluating a field. Guess you only want to build your
+        tree for specific django-apps::
+
+            >>> class MyModelTree(ModelTree):
+            ...    FOLLOW_ACROSS_APPS = True
+            ...    def _follow(self, field):
+            ...       if field.related_model._meta.app_label in ['testapp']:
+            ...          return True
+            ...       else:
+            ...          return False
+            ... 
+            >>> tree = MyModelTree(ModelOne)
+            >>> all(n.model._meta.app_label == 'testapp' for n in tree.iterate())
+            True
+
+        :param field: the field of :attr:`.model` to follow building the tree
+        :type field: a model's relation field
+        :return boolean: True to follow, False to not follow
+        """
+        return True
+
     def _follow_this_path(self, field):
         allowed_paths = set()
         for path in self.FIELD_PATHS:
@@ -528,16 +481,40 @@ class ModelTree(AnyNode):
         return this_path in allowed_paths
 
     def _follow_this_field(self, field):
+        # Only follow relational fields.
         if not field.is_relation:
             return False
+
+        # Do not follow a field back to its remote field.
         elif field.remote_field is self.field:
             return False
-        elif self.RELATION_TYPES and not any(getattr(field, t) for t in self.RELATION_TYPES):
+
+        # Do not follow generic relations.
+        elif (not field.related_model or
+              field.related_model._meta.app_label == 'contenttypes'):
             return False
-        elif self.FIELD_TYPES and not any(isinstance(field, t) for t in self.FIELD_TYPES):
+
+        # Do not follow across apps if not setup so.
+        elif (not self.FOLLOW_ACROSS_APPS and
+              not field.related_model._meta.app_label == self.model._meta.app_label):
             return False
+
+        # Only follow specific relation-types.
+        elif not any(getattr(field, t) for t in self.RELATION_TYPES):
+            return False
+
+        # Only follow specific field-types.
+        elif not type(field) in self.FIELD_TYPES:
+            return False
+
+        # Only follow specific field-paths.
         elif self.FIELD_PATHS and not self._follow_this_path(field):
             return False
+
+        # Allow customizing the tree building by follow method.
+        elif not self._follow(field):
+            return False
+
         else:
             return True
 
